@@ -1,11 +1,18 @@
 import datetime
 import json
 import logging
+import warnings
 from collections import namedtuple
 from copy import deepcopy
 
+from dateutil.parser import parse
+from IPython import get_ipython
+from marshmallow import Schema, ValidationError, fields, post_load, pre_dump
+
 from great_expectations import __version__ as ge_version
+from great_expectations.core.data_context_key import DataContextKey
 from great_expectations.core.id_dict import IDDict
+from great_expectations.core.urn import ge_urn
 from great_expectations.core.util import nested_update
 from great_expectations.exceptions import (
     InvalidCacheValueError,
@@ -15,8 +22,6 @@ from great_expectations.exceptions import (
     UnavailableMetricError,
 )
 from great_expectations.types import DictDot
-from IPython import get_ipython
-from marshmallow import Schema, ValidationError, fields, post_load, pre_dump
 
 logger = logging.getLogger(__name__)
 
@@ -104,6 +109,7 @@ def convert_to_json_serializable(data):
             ExpectationSuite,
             ExpectationValidationResult,
             ExpectationSuiteValidationResult,
+            RunIdentifier,
         ),
     ):
         return data.to_json_dict()
@@ -223,6 +229,7 @@ def ensure_json_serializable(data):
             ExpectationSuite,
             ExpectationValidationResult,
             ExpectationSuiteValidationResult,
+            RunIdentifier,
         ),
     ):
         return
@@ -307,6 +314,86 @@ def ensure_json_serializable(data):
         )
 
 
+class RunIdentifier(DataContextKey):
+    """A RunIdentifier identifies a run (collection of validations) by run_name and run_time."""
+
+    def __init__(self, run_name=None, run_time=None):
+        super(RunIdentifier, self).__init__()
+        assert run_name is None or isinstance(
+            run_name, str
+        ), "run_name must be an instance of str"
+        assert run_time is None or isinstance(run_time, (datetime.datetime, str)), (
+            "run_time must be either None or " "an instance of str or datetime"
+        )
+        self._run_name = run_name
+
+        if isinstance(run_time, str):
+            try:
+                run_time = parse(run_time)
+            except ParserError:
+                warnings.warn(
+                    f'Unable to parse provided run_time str ("{run_time}") to datetime. Defaulting '
+                    f"run_time to current time."
+                )
+                run_time = datetime.datetime.now(datetime.timezone.utc)
+
+        run_time = run_time or datetime.datetime.now(datetime.timezone.utc)
+        if not run_time.tzinfo:
+            # this takes the given time and just adds timezone (no conversion)
+            run_time = run_time.replace(tzinfo=datetime.timezone.utc)
+        else:
+            # this takes given time and converts to utc
+            run_time = run_time.astimezone(tz=datetime.timezone.utc)
+        self._run_time = run_time
+
+    @property
+    def run_name(self):
+        return self._run_name
+
+    @property
+    def run_time(self):
+        return self._run_time
+
+    def to_tuple(self):
+        return (
+            self._run_name or "__none__",
+            self._run_time.strftime("%Y%m%dT%H%M%S.%fZ"),
+        )
+
+    def to_fixed_length_tuple(self):
+        return (
+            self._run_name or "__none__",
+            self._run_time.strftime("%Y%m%dT%H%M%S.%fZ"),
+        )
+
+    def __repr__(self):
+        return json.dumps(self.to_json_dict())
+
+    def __str__(self):
+        return json.dumps(self.to_json_dict(), indent=2)
+
+    def to_json_dict(self):
+        myself = runIdentifierSchema.dump(self)
+        return myself
+
+    @classmethod
+    def from_tuple(cls, tuple_):
+        return cls(tuple_[0], tuple_[1])
+
+    @classmethod
+    def from_fixed_length_tuple(cls, tuple_):
+        return cls(tuple_[0], tuple_[1])
+
+
+class RunIdentifierSchema(Schema):
+    run_name = fields.Str()
+    run_time = fields.DateTime(format="iso")
+
+    @post_load
+    def make_run_identifier(self, data, **kwargs):
+        return RunIdentifier(**data)
+
+
 class ExpectationKwargs(dict):
     ignored_keys = ["result_format", "include_config", "catch_exceptions"]
 
@@ -571,7 +658,10 @@ class ExpectationSuite(object):
             self.meta["citations"] = []
         self.meta["citations"].append(
             {
-                "citation_date": citation_date or datetime.datetime.now().isoformat(),
+                "citation_date": citation_date
+                or datetime.datetime.now(datetime.timezone.utc).strftime(
+                    "%Y%m%dT%H%M%S.%fZ"
+                ),
                 "batch_kwargs": batch_kwargs,
                 "batch_markers": batch_markers,
                 "batch_parameters": batch_parameters,
@@ -1290,3 +1380,4 @@ expectationConfigurationSchema = ExpectationConfigurationSchema()
 expectationSuiteSchema = ExpectationSuiteSchema()
 expectationValidationResultSchema = ExpectationValidationResultSchema()
 expectationSuiteValidationResultSchema = ExpectationSuiteValidationResultSchema()
+runIdentifierSchema = RunIdentifierSchema()
